define([
    'jquery',
    'underscore',
    'backbone',
    'view/file_drop_view',
    'view/mt_dialog',
    'view/mt_modal',
    'model/upload_file',
    'model/upload_file_collection',
    'view/upload_view',
    'multi_upload_settings',
    'view/custom_field_file_drop_view',
    'view/debug_view'
],
function(
    $,
    _,
    Backbone,
    FileDropView,
    MTDialog,
    MTModal,
    UploadFile,
    UploadFileCollection,
    UploadView,
    MultiUploadSettings,
    CustomFieldFileDropView,
    DebugView
) {
    'use strict';
    var App = Backbone.View.extend({
        el: 'body',
        initialize: function() {
            this.initDialogElement();
            this.initDroppableZone();
            // this.dropView = new FileDropView({ enabled: true });
            // this.dialog = new MTDialog();
            this.dialog = new MTModal();

            Backbone.on('fileDropped', this.fileDropped, this);
            Backbone.on('close:dialog', this.closeDialog, this);
            $(document).on('iuu:open:dialog', _.bind(this.openDragDropDialog, this));
            this.dropViews = {};
        },
        render: function() {
            var self = this;
            $('.droppable').each(function() {
                var id = $(this).prop('id');
                if(!self.dropViews[id]) {
                    var fileDropViewClass = FileDropView;
                    if(id.match(/^customfield_/)) {
                        fileDropViewClass = CustomFieldFileDropView;
                    }
                    self.dropViews[id] = new fileDropViewClass({
                        enabled: true,
                        editField: id
                    });
                }
                iuu.log('[IUU] ', id);
                if(id.match(/^customfield_/)) {
                    iuu.log('[IUU] custom_fields');
                    $(this).find('.mt-draggable__content').prepend(self.dropViews[id].render().$el);
                }
                else {
                    $(this).prepend(self.dropViews[id].render().$el);
                }
            });
            return this;
        },
        _initDialogElement: function() {
            var $dialog = $('.mt-dialog');
            if($dialog.length === 0) {
                this.$el.append('<div class="mt-dialog"><span>Close</span></div>');
                this.$('.mt-dialog').after('<div class="mt-dialog-overlay overlay" />');
            }
        },
        initDialogElement: function () {
            var $dialog = $('.mt-modal');
            if($dialog.length === 0) {
                var html = '<style>iframe.embed-response-item:not(:last-child) { display: none; }</style>'
                    + '<div class="modal fade mt-modal">'
                    + '<div class="modal-dialog">'
                    + '<div class="modal-content embed-responsive">'
                    + '</div>'
                    + '</div>'
                    + '</div>';
                this.$el.append(html);
            }
        },
        initDroppableZone: function() {
            var custom_fields = MultiUploadSettings.custom_fields;
            if(!custom_fields) {
                custom_fields = [];
            }
            custom_fields.push('text');
            _.each(custom_fields, function(field_id) {
                var field = $('#' + field_id + '-field');
                field.addClass('droppable');
                if(field_id.match(/^customfield_/)) {
                    field.addClass('customfield');
                }
            });

            var content_fields = MultiUploadSettings.content_fields;
            if(!content_fields) {
                content_fields = [];
            }
            _.each(content_fields, function(field_id) {
                var field = $('#' + field_id);
                field.addClass('droppable');
            });
        },
        getEditField: function(editField) {
            var field = '';
            var match = null;
            if(editField === 'text-field') {
                field = window.app.editor.id.toString();
            }
            else if(editField.match(/^customfield_/)) {
                field = editField.replace(/^(.+)-field$/, '$1');
            }
            else if(match = editField.match(/^contentblock(\d+)/)) {
                field = 'editor-input-content-field-' + match[1];
            }
            else if(match = editField.match(/^asset-field-\d+$/)) {
                field = editField;
            }
            return field;
        },
        // Called when files are dropped on the editor or the content field directly.
        fileDropped: function(source, files, opts) {
            if(source.onDialog) {
                return;
            }
            if(files.length === 0) {
                return;
            }
            iuu.log('droppedSource: ', source.editField);
            if(source.editField.match(/^customfield_/)) {
                // カスタムフィールドは1つだけdrop可。残りは無視。
                files = [ files[0] ];
            }
            /*
            else if(source.editField.match(/^asset-field-/) && opts) {
                // 画像コンテンツフィールドの最大数を超えないようにdropする。
                var remainings = opts.max - opts.selected;
                if(remainings <= 0) {
                    iuu.log('Max files has been exceeded.');
                    return;
                }
                else {
                    files = Array.from(files).slice(0, remainings);
                }
            }
            */

            // sort by file's name
            if(MultiUploadSettings.sort_files) {
                files = _.sortBy(files, 'name');
            }

            var skipFiles = 0;
            if(!this.fileCollection) {
                this.fileCollection = new UploadFileCollection();
            }
            _.each(files, function(file) {
                iuu.log(file);
                if(!file.type.match(/^image\//)) {
                    iuu.log('not image, skip: ', file.type);
                    skipFiles++;
                    return;
                }
                var uploadFile = new UploadFile({
                    file: file,
                    editField: this.getEditField(source.editField)
                });
                iuu.log('uploadFile:', uploadFile);
                this.fileCollection.add(uploadFile);
            }, this);

            var uploadView = new UploadView({ model: this.fileCollection, editField:source.editField });
            this.dialog.empty();
            this.dialog.append(uploadView.render().$el);
            this.dialog.open();

            if(skipFiles > 0) {
                _.delay(function() {
                    alert('画像ファイル以外はアップロードできません。');
                }, 1000);
            }
        },
        closeDialog: function() {
            iuu.log('app: closeDialog');
            this.dialog.close();
            this.fileCollection.cleanUp();
            this.fileCollection.reset();
        },
        openDragDropDialog: function() {
            iuu.log('iuu:open:dialog');
            if(!this.fileCollection) {
                this.fileCollection = new UploadFileCollection();
            }
            var dialogUploadView = new UploadView({ model: this.fileCollection, droppable: true });
            this.dialog.empty();
            this.dialog.append(dialogUploadView.render().$el);
            this.dialog.open();
        }
    });
    return App;
});

