define([
    'jquery',
    'underscore',
    'backbone',
    'text!tmpl/drop_view.html',
    'multi_upload_settings'
],
function($, _, Backbone, tmpl, MultiUploadSettings) {
    'use strict';
    var CustomFieldFileDropView = Backbone.View.extend({
        className: 'dropzone dropzone-border customfield field',
        template: _.template(tmpl),
        overContainer: false,
        overDropZone: false,
        localDrag: false,
        draggingFile: null,
        enabled: false,
        editField: '',
        onDialog: false,
        customFieldId: '',
        events: {
            'drop': 'dropOnDropZone'
        },
        initialize: function(options) {
            if(options.enabled) {
                this.enabled = options.enabled;
            }
            if(options.editField) {
                this.editField = options.editField;
                this.customFieldId = options.editField.replace('-field', '');
            }
            else {
                iuu.log('[IUU] Missing arg, editField');
                throw new Error('[IUU] Missing arg, editField');
            }
            this.$el.append(this.template());

            this.$doc = $(document);

            this.$doc.on('dragover', _.bind(this.dragOverOnContainer, this));
            this.$doc.on('dragleave', _.bind(this.dragLeaveFromContainer, this));
            // this.$doc.on('dragover', this.el, _.bind(this.dragOverOnDropZone, this));
            // this.$doc.on('dragleave', this.el, _.bind(this.dragLeaveFromDropZone, this));
            this.$el.on('dragover', _.bind(this.dragOverOnDropZone, this));
            this.$el.on('dragleave', _.bind(this.dragLeaveFromDropZone, this));
            this.$doc.on( 'dragstart dragend drop', _.bind(function( event ) {
                this.localDrag = event.type === 'dragstart';
                iuu.log('local: ' + this.localDrag);
            }, this));

            this.toggleDropZone();

            var self = this;
            $('#' + this.customFieldId + '_remove_asset').click(function() {
                self.toggleDropZone();
            });

            Backbone.on('dialog:open', this.disable, this);
            Backbone.on('dialog:close', this.enable, this);
            Backbone.on('dropped', this.dropped, this);
            Backbone.on('file:inserted', this.inserted, this);

            $('#'+this.editField).change(_.bind(function(event) {
                this.toggleDropZone();
            }, this));
            
            if(MultiUploadSettings.permission_denied) {
                this.disable();
            }
        },
        render: function() {
            this.updateSize();
            return this;
        },
        updateSize: function() {
           var customfield = this.$el.parent().parent().parent();
           iuu.log('[IUU] customfield id: ', customfield.prop('id'));
        },
        hasFile: function() {
            return $('#'+this.customFieldId).val() ? true : false;
        },
        inserted: function() {
            iuu.log('[IUU] file inserted');
            this.toggleDropZone();
        },
        toggleDropZone: function() {
            if(this.hasFile()) {
                iuu.log('[IUU] has file, ', this.customFieldId);
                this.$el.addClass('hidden d-none');
            }
            else {
                this.$el.removeClass('hidden d-none');
            }
        },
        enable: function() {
            this.enabled = true;
            this.draggingFile = null;
            this.overContainer = false;
            this.overDropZone = false;
            iuu.log('enable');
            this.toggleDropZone();
        },
        disable: function() {
            this.enabled = false;
            this.draggingFile = null;
            this.overContainer = false;
            this.overDropZone = false;
            iuu.log('disable');
        },
        showPreview: function() {
            if(this.hasFile()) {
                var preview = $('#' + this.editField +  ' .customfield_preview a img');
                preview.show();
            }
        },
        hidePreview: function() {
            if(this.hasFile()) {
                var preview = $('#' + this.editField +  ' .customfield_preview a img');
                preview.hide();
            }
        },
        toggleHover: function() {
            if(this.overDropZone) {
                this.$el.addClass('hover');
            }
            else {
                this.$el.removeClass('hover');
            }
        },
        update: function() {
            // if(this.hasFile && (this.overContainer || this.overDropZone)) {
            if(this.overContainer || this.overDropZone) {
                this.$el.removeClass('hidden');
                if(this.hasFile()) {
                    this.hidePreview();
                }
                // iuu.log('[IUU] update hover');
            }
            else {
                if(this.hasFile()) {
                    this.$el.addClass('hidden');
                    this.showPreview();
                }
                // iuu.log('[IUU] update hover removed');
            }

            this.toggleHover();

            if(!this.overContainer || !this.overDropZone) {
                this.draggingFile = null;
            }
        },
        isDraggingFile: function(event) {
            if ( this.draggingFile !== null ) {
                return this.draggingFile;
            }
            if ( _.isUndefined( event.originalEvent ) || _.isUndefined( event.originalEvent.dataTransfer ) ) {
                return false;
            }
            this.draggingFile = _.indexOf( event.originalEvent.dataTransfer.types, 'Files' ) > -1 &&
                _.indexOf( event.originalEvent.dataTransfer.types, 'text/plain' ) === -1;
            return this.draggingFile;
        },
        cancelEvent: function(event) {
            event.stopPropagation();
            event.preventDefault();
        },
        dragOverOnContainer: function(event) {
            if(!this.enabled || this.localDrag || !this.isDraggingFile(event)) {
                return;
            }
            this.overContainer = true;
            this.update();
            // iuu.log('[IUU] Drag Over On Container');

            return false;
        },
        dragLeaveFromContainer: function(/* event */) {
            this.overContainer = false;
            _.delay(_.bind(this.update, this), 10);
            iuu.log('[IUU] Drag Leave From Container');
            return false;
        },
        dragOverOnDropZone: function(event) {
            if(!this.enabled || this.localDrag || !this.isDraggingFile(event)) {
                return;
            }
            this.overDropZone = true;
            this.update();
            return false;
        },
        dragLeaveFromDropZone: function(/* event */) {
            this.overDropZone = false;
            _.delay(_.bind(this.update, this), 10);
        },
        dropOnDropZone: function(event) {
            if(!this.enabled) {
                return false;
            }
            iuu.log('dropped', this.editField);
            Backbone.trigger('dropped', this);

            this.dragLeaveFromDropZone(event);
            this.dragLeaveFromContainer(event);
            this.cancelEvent(event);
            this.showPreview();

            var files = event.originalEvent.dataTransfer.files;
            iuu.log(files);
            if(files.length === 0) {
                return false;
            }

            var self = this;
            _.delay(function() {
                Backbone.trigger('fileDropped', self, files);
            }, 100);

            return false;
        },
        dropped: function(source) {
            if(source && source !== this) {
                this.dragLeaveFromDropZone();
                this.dragLeaveFromContainer();
            }
        }
    });
    return CustomFieldFileDropView;
});


