define([
    'jquery',
    'underscore',
    'backbone',
    'text!tmpl/drop_view.html',
    'multi_upload_settings'
],
function($, _, Backbone, tmpl, MultiUploadSettings) {
    'use strict';
    var FileDropView = Backbone.View.extend({
        className: 'dropzone dropzone-border field',
        template: _.template(tmpl),
        overContainer: false,
        overDropZone: false,
        localDrag: false,
        draggingFile: null,
        enabled: false,
        editField: '',
        onDialog: false,
        events: {
            'drop': 'dropOnDropZone'
        },
        inContainer: 0,
        inDropZone: 0,
        initialize: function(options) {
            if(options.enabled) {
                this.enabled = options.enabled;
            }
            if(options.editField) {
                this.editField = options.editField;
            }
            this.$el.append(this.template());
            this.$el.hide();
            this.$doc = $(document);
            this.$doc.on('dragenter', _.bind(this.dragEnterOnContainer, this));
            this.$doc.on('dragover', _.bind(this.dragOverOnContainer, this));
            this.$doc.on('dragleave', _.bind(this.dragLeaveFromContainer, this));
            // this.$doc.on('dragover', this.el, _.bind(this.dragOverOnDropZone, this));
            // this.$doc.on('dragleave', this.el, _.bind(this.dragLeaveFromDropZone, this));
            this.$el.on('dragenter', _.bind(this.dragEnterOnDropZone, this));
            this.$el.on('dragover', _.bind(this.dragOverOnDropZone, this));
            this.$el.on('dragleave', _.bind(this.dragLeaveFromDropZone, this));
            this.$doc.on( 'dragstart dragend drop', _.bind(function( event ) {
                this.localDrag = event.type === 'dragstart';
                // iuu.log('local: ' + this.localDrag);
            }, this));

            Backbone.on('dialog:open', this.disable, this);
            Backbone.on('dialog:close', this.enable, this);
            Backbone.on('dropped', this.dropped, this);

            if(MultiUploadSettings.permission_denied) {
                this.disable();
            }
        },
        render: function() {
            return this;
        },
        enable: function() {
            iuu.log('[IUU] file_drop_view: enable');
            this.enabled = true;
            this.draggingFile = null;
            this.overContainer = false;
            this.overDropZone = false;
        },
        disable: function() {
            iuu.log('[IUU] file_drop_view: disable');
            this.enabled = false;
            this.draggingFile = null;
            this.overContainer = false;
            this.overDropZone = false;
        },
        toggleHover: function() {
            if(this.overDropZone) {
                this.$el.addClass('hover');
            }
            else {
                this.$el.removeClass('hover');
            }
        },
        update: function() {
            // if(this.overContainer || this.overDropZone) {
            if(this.inContainer > 0 || this.overDropZone) {
                var h = this.$el.parent().height();
                var w = this.$el.parent().width();
                this.$el.outerHeight(h);
                this.$el.outerWidth(w);
                this.$el.css('position', 'absolute');
                this.$el.show();
            }
            else {
                this.$el.hide();
            }

            this.toggleHover();
            if(this.overDropZone) {
                // iuu.log('overDropZone');
            }

            // if(!this.overContainer || !this.overDropZone) {
            if(this.inContainer === 0 || !this.overDropZone) {
                this.draggingFile = null;
            }
        },
        isDraggingFile: function(event) {
            if ( this.draggingFile !== null ) {
                return this.draggingFile;
            }
            if ( _.isUndefined( event.originalEvent ) || _.isUndefined( event.originalEvent.dataTransfer ) ) {
                return false;
            }
            this.draggingFile = _.indexOf( event.originalEvent.dataTransfer.types, 'Files' ) > -1 &&
                _.indexOf( event.originalEvent.dataTransfer.types, 'text/plain' ) === -1;
            return this.draggingFile;
        },
        isDialogOpen: function() {
            if($('.mt-modal').css('display') == 'none') {
                return false;
            }
            return true;
        },
        cancelEvent: function(event) {
            event.stopPropagation();
            event.preventDefault();
        },
        isBlockEditor: function() {
            if($('#convert_breaks').val() === 'block_editor') {
                return true;
            }
            return false;
        },
        dragEnterOnContainer: function(event) {
            iuu.log('[IUU] dragenter on container', event);
            if(!this.enabled
                || this.localDrag
                || !this.isDraggingFile(event)
                || this.isDialogOpen()
                || this.isBlockEditor()) {
                return;
            }
            this.overContainer = true;
            if(this.inContainer < 0) {
                this.inContainer = 0;
            }
            this.inContainer++;
            this.update();
            return false;
        },
        dragOverOnContainer: function(event) {
            // iuu.log('[IUU] dragover on container in');
            if(!this.enabled
                || this.localDrag
                || !this.isDraggingFile(event)
                || this.isDialogOpen()
                || this.isBlockEditor()) {
                return;
            }
            this.overContainer = true;
            this.update();
            // iuu.log('[IUU] dragover on container out');
            return false;
        },
        dragLeaveFromContainer: function(event) {
            this.overContainer = false;
            this.inContainer--;
            iuu.log('[IUU] dragleave from container, inContainer=' + this.inContainer + ', inDropZone=' + this.inDropZone + ', overContainer=' + this.overContainer + ', overDropZone=' + this.overDropZone);
            if(this.inContainer <= 0) {
                iuu.log('[IUU] dragleave from container');
                this.resetFlags();
                _.delay(_.bind(this.update, this), 10);
            }
            // this.update();
            // iuu.log('[IUU] dragleave from container');
        },
        dragEnterOnDropZone: function(event) {
            iuu.log('[IUU] dragenter on dropzone', event);
            this.overDropZone = true;
            this.inContainer++;
            this.inDropZone++;
            this.update();
            return false;
        },
        dragOverOnDropZone: function(event) {
            iuu.log('[IUU] dragover on dropzone');
            if(!this.enabled || this.localDrag || !this.isDraggingFile(event) || this.isDialogOpen()) {
                return;
            }
            this.overDropZone = true;
            this.update();
            // iuu.log('[IUU] dragover on dropzone');
            return false;
        },
        dragLeaveFromDropZone: function(event) {
            this.overDropZone = false;
            this.inContainer--;
            this.inDropZone--;

            iuu.log('[IUU] dragleave from dropzone', event)
            iuu.log('[IUU] dragleave from dropzone, inContainer=' + this.inContainer + ', inDropZone=' + this.inDropZone + ', overContainer=' + this.overContainer + ', overDropZone=' + this.overDropZone);

            if(event) {
                // iuu.log('[IUU] dragleave from dropzone, target='+event.target.tagName+', currentTarget='+event.currentTarget.tagName);
                // iuu.log('[IUU] dragleave from dropzone', event);
                // if((event.target === event.currentTarget)
                //     || (event.currentTarget === this.$el.get(0))) {
                //     return false;
                // }
                if(this.isInside(event)) {
                    iuu.log('[IUU] dragleave from dropzone, but still inside', event);
                    return false;
                }
                else if(this.isOutOfBrowser(event)) {
                    iuu.log('[IUU] dragleave from dropzone, but out of browser', event);
                    this.inContainer = 0;
                    this.inDropZone = 0;
                    this.overContainer = false;
                    this.overDropZone = false;
                }
            }
            // this.update();
            _.delay(_.bind(this.update, this), 10);
            iuu.log('[IUU] dragleave from dropzone');
            return false;
        },
        isInside: function(e) {
            var rect = e.currentTarget.getBoundingClientRect();
            var x = e.clientX;
            var y = e.clientY;
            return (rect.left <= x && x <= rect.right
                && rect.top <= y && y <= rect.bottom);
        },
        isOutOfBrowser: function(e) {
            return (e.clientX <= 0 && e.clientY <= 0);
        },
        resetFlags: function() {
            this.inContainer = 0;
            this.inDropZone = 0;
            this.overContainer = false;
            this.overDropZone = false;
        },
        dropOnDropZone: function(event) {
            if(!this.enabled) {
                return false;
            }
            // iuu.log('dropped', this.editField);
            Backbone.trigger('dropped', this);

            // this.dragLeaveFromDropZone(event);
            // this.dragLeaveFromContainer(event);
            this.resetFlags();
            this.cancelEvent(event);
            _.delay(_.bind(this.update, this), 10);

            var files = event.originalEvent.dataTransfer.files;
            // iuu.log(files);
            if(files.length === 0) {
                return false;
            }

            var self = this;
            var opts = this.getContentFieldOptions();
            _.delay(function() {
                Backbone.trigger('fileDropped', self, files, opts);
            }, 100);

            return false;
        },
        dropped: function(source) {
            if(source && source !== this) {
                this.dragLeaveFromDropZone();
                this.dragLeaveFromContainer();
            }
        },
        getContentFieldOptions: function() {
            if(!(this.editField && this.editField.match(/^asset-field-/))) {
                return null;
            }
            var $editor = $('#'+this.editField);
            var data = $editor.data();
            var opts = {
                'min': data.mtMinSelect || 1,
                'max': data.mtMaxSelect || 1,
                'multiple': data.mtMultiple || 0,
                'selected': $editor.find('.mt-draggable').length,
            };
            return opts;
        }
    });
    return FileDropView;
});

