define([
    'jquery',
    'underscore',
    'backbone',
    'model/upload_file',
    'model/upload_file_collection',
    'view/upload_file_view'
],
function($, _, Backbone, UploadFile, UploadFileCollection, UploadFileView) {
    'use strict';
    var UploadFileCollectionView = Backbone.View.extend({
        className: 'upload-file-options-list sortable ui-sortable',
        initialize: function() {
            var self = this;
            this.$el.sortable({
                items: 'div.sortable-item',
                placeholder: 'placeholder',
                distance: 3,
                opacity: 0.8,
                cursor: 'move',
                forcePlaceholderSize: true,
                handle: '.sort-handle',
                // containment: 'document',
                containment: 'parent',
                update: _.bind(self.updateOrder, self)
            });
            this.listenTo(this.model, 'update', this.update);
            this.listenTo(this.model, 'destroy', this.destroy);
            Backbone.on('close:dialog', this.closeDialog, this);
        },
        render: function() {
            this.model.each(function(file) {
                var fileView = new UploadFileView({ model: file });
                this.$el.append(fileView.render().$el);
            }, this);
            this.updateVisibility();
            return this;
        },
        updateVisibility: function () {
            if(this.model.length == 0) {
                this.$el.hide();
            }
            else {
                this.$el.show();
            }
        },
        update: function() {
            iuu.log('model updated: len: ' + this.model.length);
            this.updateVisibility();
        },
        destroy: function() {
            iuu.log('model destroyed: len: ' + this.model.length);
        },
        closeDialog: function() {
            this.remove();
        },
        updateOrder: function() {
            iuu.log('order changed');
            Backbone.trigger('update:fileOrder');
            this.model.sort();
        }
    });
    return UploadFileCollectionView;
});

