define([
    'jquery',
    'underscore',
    'backbone',
    'text!tmpl/upload_progress.html',
    'multi_upload_settings'
],
function($, _, Backbone, tmpl, MultiUploadSettings) {
    'use strict';
    var UploadProgress = Backbone.View.extend({
        className: 'modal fade',
        template: _.template(tmpl),
        events: {
        },
        max: 1,
        current: 0,
        initialize: function(options) {
            if(options.max) {
                this.max = options.max;
            }
            this.current = 0;

            Backbone.on('upload:file:done', _.bind(this.updateProgress, this));
            Backbone.on('upload:finish', _.bind(this.finishProgress, this));
        },
        render: function() {
            this.$el.append(this.template());
            this.$progressbar = this.$('div.progress-bar');
            this.$el.on('shown.bs.modal', _.bind(this.shown, this));
            if(MultiUploadSettings.admin_theme_id === 'admin2023') {
                var $bsmodal = new bootstrap.Modal(this.$el[0], { backdrop: 'static' });
                $bsmodal.show();
            }
            else {
                this.$el.modal({ backdrop: 'static' });
            }
            return this;
        },
        shown: function() {
            this.$el.css('z-index', 3001);
            var modal = this.$el.data('bs.modal');
            if(!modal) return;
            var $backdrop = $(modal._backdrop);
            $backdrop.css('z-index', 3000);
        },
        updateProgress: function() {
            this.current++;
            var percent = Math.round(this.current / this.max * 100);
            this.$progressbar.text(percent + '%');
            this.$progressbar.attr({ 'aria-valuenow': percent });
            this.$progressbar.css('width', percent + '%');
        },
        finishProgress: function() {
            this.$progressbar.text('100%');
            _.delay(
                _.bind(function() {
                    this.close();
                }, this),
                1000
            );
        },
        close: function() {
            this.$el.modal('hide');
            _.delay(
                _.bind(function () {
                    this.remove();
                }, this),
                500
            );
        },
        remove: function () {
            this.$el.modal('dispose');
            this.$el.remove();
        }
    });
    return UploadProgress;
});

