# ImageUploadUtility - (c) 2012-2017 Juxtaposition. All Rights Reserved.
# This code cannot be redistributed without permission from juxtaposition.jp
# For more information, consult your ImageUploadUtility license.
#

package MT::Plugin::ImageUploadUtility;

use strict;
use warnings;
use Carp;
use MT::Plugin;
use base qw( MT::Plugin );

our $PLUGIN_NAME = 'ImageUploadUtility Pro(Trial)';
our $VERSION = '3.20';

use MT;
my $plugin = MT::Plugin::ImageUploadUtility->new({
    id => 'ImageUploadUtility',
    key => __PACKAGE__,
    name => "<__trans phrase='$PLUGIN_NAME'>",
    version => $VERSION,
    description => "<__trans phrase='This plugin allows you to resize uplodaed image and to convert multibyte file name.'>",
    author_name => 'Juxtaposition',
    author_link => 'http://www.lat43n.com/',
    plugin_link => 'http://skeleton.juxtaposition.jp/image-upload-utility/',
    doc_link => 'http://skeleton.juxtaposition.jp/image-upload-utility/',
    l10n_class => 'ImageUploadUtility::L10N',
    blog_config_template => 'blog_config.tmpl',
    system_config_template => \&system_config_template,
    settings => new MT::PluginSettings([
        ['image_upload_utility_enable_resize',           { Default => 0 }],
        ['image_upload_utility_max_size',                { Default => 960 }],
        ['image_upload_utility_enable_rename',           { Default => 0 }],
        ['image_upload_utility_rename_format',           { Default => '<mt:date format="%Y%m%d%H%M%S" />-<mt:var name="random_number" />' }],
        ['image_upload_utility_strip_exif',              { Default => 0 }],
        ['image_upload_utility_watermark_asset_id',      { Default => 0 }],
        ['image_upload_utility_watermark_position',      { Default => 'SouthEast' }],
        ['image_upload_utility_watermark_opacity',       { Default => 100 }],
        ['image_upload_utility_watermark_always_use',    { Default => 1 }],
        ['image_upload_utility_max_width',               { Default => 960 }],
        ['image_upload_utility_ext_lowercase',           { Default => 0 }],
        ['image_upload_utility_jpeg_quality',            { Default => '' }],
        ['image_upload_utility_enable_pdf_thumbnail',    { Default => 0, Scope => 'system' }],
        ['image_upload_utility_normalize_orientation',   { Default => 1 }],
        ['image_upload_utility_sort_files',              { Default => 0 }],
        ['image_upload_utility_enclose_start_tag',       { Default => '<p>' }],
        ['image_upload_utility_enclose_end_tag',         { Default => '</p>' }],
    ]),
});

MT->add_plugin($plugin)
    if is_mt70_or_later();

sub instance { $plugin; }

use constant ENABLE_EXTRA => 1;

BEGIN {
    my $has_plugin_extra = eval { require ImageUploadUtility::Plugin::Extra; 1 } ? 1 : 0;
    sub has_extra {
        return ENABLE_EXTRA && is_mt70_or_later() && $has_plugin_extra;
    }
    sub has_plugin_extra {
        return $has_plugin_extra;
    }
}
    
sub is_pdf_supported {
    return has_extra() && ImageUploadUtility::Plugin::Extra::is_pdf_supported();
}

sub get_image_driver {
    my $plugin = shift;
    my $app = MT->instance
        or die  "failed to get app instance";
    my $graphic_drv = '';
    my $image_driver = $app->config('ImageDriver') || '';
    if($image_driver eq 'ImageMagick') {
        eval { require Image::Magick; };
        die "failed to load Image Driver" if $@;
        $graphic_drv = 'Image::Magick';
    }
    elsif($image_driver eq 'GraphicsMagick') {
        eval { require Graphics::Magick; };
        die "failed to load Image Driver" if $@;
        $graphic_drv = 'Graphics::Magick';
    }
    die "ImageDriver is not configured" unless $graphic_drv;
    return $graphic_drv;
}

sub init_registry {
    my $plugin = shift;
    $plugin->SUPER::init_registry;
    my $registry = {
        callbacks =>  {
            'init_app'
                => '$ImageUploadUtility::ImageUploadUtility::Plugin::init_app',
            'template_source.edit_entry'
                => '$ImageUploadUtility::ImageUploadUtility::Plugin::edit_entry',
            'template_param.edit_entry'
                => '$ImageUploadUtility::ImageUploadUtility::Plugin::edit_param',
            'template_source.async_asset_upload' 
                => '$ImageUploadUtility::ImageUploadUtility::Plugin::source_async_asset_upload',
            'template_param.async_asset_upload' 
                => '$ImageUploadUtility::ImageUploadUtility::Plugin::param_async_asset_upload',
            'template_param.asset_modal' 
                => '$ImageUploadUtility::ImageUploadUtility::Plugin::param_asset_modal',
            'template_param.asset_upload'
                => '$ImageUploadUtility::ImageUploadUtility::Plugin::param_asset_upload',
            'template_source.edit_content_data'
                => '$ImageUploadUtility::ImageUploadUtility::Plugin::edit_entry',
            'template_param.edit_content_data'
                => '$ImageUploadUtility::ImageUploadUtility::Plugin::edit_param',
        },
        tags => {
            modifier => {
                'image_max_size' => sub {
                    require ImageUploadUtility::Plugin;
                    ImageUploadUtility::Plugin::fltr_image_max_size(@_);
                },
                'image_max_width' => sub {
                    require ImageUploadUtility::Plugin;
                    ImageUploadUtility::Plugin::fltr_image_max_width(@_);
                },
            },
        },
    };
            
    if($plugin->has_extra) {
        $registry = ImageUploadUtility::Plugin::Extra::merge_registry($plugin, $registry);
    }

    $plugin->registry($registry);

    return 1;
}

sub is_mt52_or_later {
    return (substr(MT->version_number, 0, 3) eq '5.2')
        || (substr(MT->version_number, 0, 1) >= 6);
}

sub is_mt62 {
    return (substr(MT->version_number, 0, 3) eq '6.2');
}

sub is_mt61_or_earlier {
    my ($major, $minor) = split(/\./, MT->version_number);
    return ($major == 6 && $minor <= 1) || ($major < 6);
}

sub is_mt62_or_later {
    my ($major, $minor) = split(/\./, MT->version_number);
    return ($major == 6 && $minor >= 2) || ($major > 6);
}

sub is_mt70_or_later {
    my ($major, $minor) = split(/\./, MT->version_number);
    return ($major == 7 && $minor >= 0) || ($major > 7);
}

sub is_upload_compat {
    return (is_mt62_or_later() && MT->config('EnableUploadCompat'))
        || is_mt61_or_earlier();
}

sub doLog {
    my ($plugin, $msg) = @_;
    $msg = $plugin unless ref $plugin;
    return unless defined($msg);

    require MT::Log;
    my $log = MT::Log->new;
    $log->message($msg) ;
    $log->save or die $log->errstr;
    return;
}

sub system_config_template {
    my $plugin = shift;
    if($plugin->has_extra) {
        return $plugin->load_tmpl('system_config.tmpl');
    }
    return;
}

sub apply_default_settings {
    my $plugin = shift;
    my ($data, $scope_id) = @_;

    my $prefix = 'IUUDefault';
    my @params = qw(
        EnableResize
        MaxSize
        EnableRename
        RenameFormat
        StripExif
        WatermarkAssetId
        WatermarkPosition
        WatermarkOpacity
        WatermarkAlwaysUse
        MaxWidth
        ExtLowercase
        JpegQuality
        NormalizeOrientation
        SortFiles
        EncloseStartTag
        EncloseEndTag
    );
    my $app = MT->instance
        or return;
    foreach my $param (@params) {
        my $value = $app->config($prefix . $param);
        if (defined $value) {
            $param =~ s!^([A-Z])!\L$1\E!;
            $param =~ s!([A-Z])!_\L$1\E!g;
            my $key = 'image_upload_utility_' . $param;
            $data->{$key} = $value if !exists $data->{$key};
        }
    }

    $plugin->SUPER::apply_default_settings($data, $scope_id);
}

sub load_config {
    my $plugin = shift;
    my ($param, $scope) = @_;
    $plugin->SUPER::load_config($param, $scope);
    if ($scope =~ m/blog:(\d+)/)  {
        my $blog_id = $1;
        $param->{blog_id} = $blog_id;
    }
    if ($plugin->has_extra) {
        $param->{image_upload_utility_has_extra} = 1;
        $param->{image_upload_utility_is_pdf_supported} = $plugin->is_pdf_supported();
    }
    $param->{image_upload_utility_has_imagemagick} = $plugin->get_image_driver() ? 1 : 0;
}

sub save_config {
    my $plugin = shift;
    my ($param, $scope) = @_;
    my $vars_scope = $scope;
    $vars_scope =~ s/:.*//;
    $vars_scope = 'system' unless $vars_scope;
    if($vars_scope ne 'system') {
        my $pattern = '^image_upload_utility_(max_size|thumbnail_max_size|rename_format|max_width|jpeg_quality|watermark_opacity)$';
        foreach my $key (keys %$param) {
            next unless $key =~ m!$pattern!;
            my $name = $1;
            my $value = $param->{$key};
            if($name =~ m!^(?:max_size|thumbnail_max_size|max_width)$!
                && !($value =~ m!^\d+$! && $value > 0)) {
                return $plugin->error(
                    $plugin->translate(
                        '([_1]) must be greater than 0.',
                        $plugin->translate($name),
                    )
                );
            }
            elsif($name =~ m!^(?:jpeg_quality|watermark_opacity)$!
                && !(defined $value && (length $value == 0 || ($value =~ m!^\d+$! && $value >= 0 && $value <= 100 )))) {
                return $plugin->error(
                    $plugin->translate(
                        '([_1]) must be empty string or integer(0-100).',
                        $plugin->translate($name),
                    ),
                );
            }
            elsif($name =~ m!^rename_format!
                && !(defined $value && length $value > 0)) {
                return $plugin->error(
                    $plugin->translate(
                        '([_1]) must be specified.',
                        $plugin->translate($name),
                    )
                );
            }
        }
    }

    if($vars_scope eq 'system') {
        if($plugin->has_extra) {
            unless($plugin->is_pdf_supported) {
                if(exists $param->{image_upload_utility_enable_pdf_thumbnail}) {
                    $param->{image_upload_utility_enable_pdf_thumbnail} = 0;
                }
            }
            my $value = exists $param->{image_upload_utility_enable_pdf_thumbnail} ? $param->{image_upload_utility_enable_pdf_thumbnail} : 0;
            my $old_value = $plugin->SUPER::get_config_value('image_upload_utility_enable_pdf_thumbnail', $scope) || 0;
            if($value != $old_value) {
                ImageUploadUtility::Plugin::Extra::toggle_asset_type($plugin, $value);
            }
        }
    }

    my $app = MT->instance;
    if($app && $app->can('reboot')) {
        $app->reboot;
    }

    return $plugin->SUPER::save_config($param, $scope);
}

sub get_config_obj {
    my $plugin = shift;
    my ($scope_id) = @_;
    my $config_obj = $plugin->SUPER::get_config_obj(@_)
        or return;
    my $config_hash = $config_obj->data 
        or return $config_obj;

    # default settings have not correctly applied when upgraded from v1.21.
    if(exists $config_hash->{image_upload_utility_watermark_always_use}
        && !defined $config_hash->{image_upload_utility_watermark_always_use}) {
        $config_hash->{image_upload_utility_watermark_always_use} = 1;
    }
    if(exists $config_hash->{image_upload_utility_watermark_position}
        && !defined $config_hash->{image_upload_utility_watermark_position}) {
        $config_hash->{image_upload_utility_watermark_position} = 'SouthEast';
    }
    $config_obj->data($config_hash);
    return $config_obj;
}

sub get_config_value {
    my $plugin = shift;
    my ($key, $blog_id) = @_;
    my $scope = 'system';
    if($blog_id) {
        $scope = 'blog:'.$blog_id;
    }
    return $plugin->SUPER::get_config_value(
        'image_upload_utility_'.$key, $scope
    );
}

sub jpeg_quality {
    my $plugin = shift;
    my ($blog_id) = @_;
    my $jpeg_quality = $plugin->get_config_value('jpeg_quality', $blog_id);
    if (defined $jpeg_quality && $jpeg_quality eq '') {
        $jpeg_quality = MT->instance->config->IUUJpegQuality;
    }
    if(defined $jpeg_quality && $jpeg_quality =~ m!^\d+$!) {
        if($jpeg_quality < 0 || $jpeg_quality > 100) {
            $jpeg_quality = undef;
        }
    }
    else {
        $jpeg_quality = undef;
    }
    return $jpeg_quality;
}

sub DESTROY {}

sub AUTOLOAD {
    my $plugin = $_[0];
    (my $method = our $AUTOLOAD) =~ s!.+::!!;
    croak("Cannot find method '$method' for __PACKAGE__")
        unless ref $plugin;
    no strict 'refs';
    *$AUTOLOAD = sub {
        my $plugin = shift;
        return $plugin->get_config_value($method, @_);
    };
    goto &$AUTOLOAD;
}

1;
__END__
